/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.gradle.plugins.target.platform.internal.util;

import groovy.lang.Closure;
import groovy.lang.GroovyObjectSupport;

import io.spring.gradle.dependencymanagement.dsl.DependencyManagementConfigurer;
import io.spring.gradle.dependencymanagement.dsl.DependencyManagementExtension;
import io.spring.gradle.dependencymanagement.dsl.ImportsHandler;

import java.util.ArrayList;
import java.util.List;

import org.gradle.api.Action;
import org.gradle.api.Project;
import org.gradle.api.artifacts.Configuration;
import org.gradle.api.artifacts.ConfigurationContainer;
import org.gradle.api.artifacts.Dependency;
import org.gradle.api.artifacts.DependencySet;

/**
 * @author Gregory Amerson
 */
public class TargetPlatformPluginUtil {

	public static void configureDependencyManagement(
		final Project project,
		final Configuration targetPlatformBomsConfiguration,
		Iterable<?> configurationNames) {

		final DependencyManagementExtension dependencyManagementExtension =
			GradleUtil.getExtension(
				project, DependencyManagementExtension.class);

		GroovyObjectSupport groovyObjectSupport =
			(GroovyObjectSupport)dependencyManagementExtension;

		List<Object> args = new ArrayList<>();

		ConfigurationContainer configurationContainer =
			project.getConfigurations();

		for (Object configurationName : configurationNames) {
			Configuration configuration = null;

			if (configurationName instanceof Configuration) {
				configuration = (Configuration)configurationName;
			}
			else {
				configuration = configurationContainer.findByName(
					GradleUtil.toString(configurationName));
			}

			if (configuration != null) {
				args.add(configuration);
			}
		}

		Closure<Void> closure = new Closure<Void>(project) {

			@SuppressWarnings("unused")
			public void doCall() {
				DependencySet dependencySet =
					targetPlatformBomsConfiguration.getAllDependencies();

				dependencySet.all(
					new Action<Dependency>() {

						@Override
						public void execute(final Dependency dependency) {
							_configureDependencyManagementImportsHandler(
								(DependencyManagementConfigurer)getDelegate(),
								dependency);
						}

					});
			}

		};

		args.add(closure);

		groovyObjectSupport.invokeMethod(
			"configurations", args.toArray(new Object[0]));
	}

	private static void _configureDependencyManagementImportsHandler(
		DependencyManagementConfigurer dependencyManagementConfigurer,
		final Dependency dependency) {

		dependencyManagementConfigurer.imports(
			new Action<ImportsHandler>() {

				@Override
				public void execute(ImportsHandler importsHandler) {
					StringBuilder sb = new StringBuilder();

					sb.append(dependency.getGroup());
					sb.append(':');
					sb.append(dependency.getName());
					sb.append(':');
					sb.append(dependency.getVersion());

					importsHandler.mavenBom(sb.toString());
				}

			});
	}

}